#define MAIN
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "polyr.h"

extern int NUM_VERTICES;
extern float *VERTICES;
extern float *NORMALS;


void
main(int argc, char *argv[])
{
  int i;
  int xdim, ydim, zdim;	/* the dimensions of the data */
  float min,max;
  float *data,*newdata;
  int size[3];
  int newsize[3];
  float threshold,dsfactor;	      
  char inp_file[80];	      
  float *downsample(float *, int *, int *);  
  
  /* get rutime name */
  strcpy(MY_NAME, argv[0]);
  printf("\n%s: <Jesper James Jensen 1995>\n",MY_NAME);

  /* no args? print help */
  if (argc == 1) {
    usage();
    exit(1);
  }
  /* last two args are input name and threshold */
  strcpy(inp_file, argv[(argc - 2)]);
  threshold = (float) atof(argv[(argc - 1)]);

  /* get command line args */
  if (parse_options(argc, argv) == -1) {
    fprintf(stderr, "%s: error from parse_options\n",MY_NAME);
    exit(1);
  }
  /* no input format specified? */
  if ((ANA_INPUT+RAW_INPUT+MS_INPUT+HDF_INPUT)!=1) {
    fprintf(stderr,"%s: missing or conflicting file type(s)\n",MY_NAME);
    exit(1);
  }
  /* no output format specified? */
  if (!STORE_POLYGONS) {
    fprintf(stderr,"%s: Output file name not specified\n",MY_NAME);
    exit(1);
  }
  /* read the array */
  if (ANA_INPUT) {
    if (get_analyze(inp_file, &data, &xdim, &ydim, &zdim, delta, &max, &min) == -1) {
      fprintf(stderr, "%s: error from get_analyze\n", MY_NAME);
      exit(1);
    }
  }
  else if (RAW_INPUT) {
    if (XDIM <= 0 || YDIM <= 0 || ZDIM <= 0) {
      fprintf(stderr, "%s: bad dimensions for raw input file\n", MY_NAME);
      exit(1);
    }
    xdim = XDIM; ydim = YDIM; zdim = ZDIM;
    if (get_raw_data(inp_file, &data, xdim, ydim, zdim, &max, &min) == -1) {
      fprintf(stderr, "%s: error from get_raw_data\n", MY_NAME);
      exit(1);
    }
  }
  else if (MS_INPUT) {
    if (get_ms_data(inp_file, &data, &xdim, &ydim, &zdim, delta, &max, &min) == -1) {
      fprintf(stderr, "%s: error from get_ms_data\n", MY_NAME);
      exit(1);
    }
  }
  else if (HDF_INPUT) {
    if (get_hdf_data(inp_file, &data, &xdim, &ydim, &zdim, &max, &min) == -1) {
      fprintf(stderr, "%s: error from get_hdf_data\n", MY_NAME);
      exit(1);
    }
  }
  
  if (VERBOSE) 
    printf("%s: Data range: [%f;%f]\n", MY_NAME, min, max);

  size[0]=xdim; size[1]=ydim; size[2]=zdim;
  
  tid1=clock();
  
  /* filtering of the input array */
  if (FILTER[1]) {
    if (VERBOSE)
      printf("%s: Mean filtering the array with a filter [%dx%dx%d]\n",
	     MY_NAME,FILTER[0],FILTER[1],FILTER[2]);
    boxfilter(&data,size,FILTER);
  }

  tid2=clock();

  /* Ok both downsampling and isosurfacing! */
  if ((DOWNSAMPL[0]!=1.0) ||(DOWNSAMPL[1]!=1.0) ||(DOWNSAMPL[2]!=1.0)) {
    if (VERBOSE)
      printf("%s: Downsampling array with x:%.2f y:%.2f z:%.2f\n",
	     MY_NAME,DOWNSAMPL[0],DOWNSAMPL[1],DOWNSAMPL[2]);
    delta[0]*=DOWNSAMPL[0]; delta[1]*=DOWNSAMPL[1]; delta[2]*=DOWNSAMPL[2];
    xdim=(newsize[0]=xdim/DOWNSAMPL[0]);
    ydim=(newsize[1]=ydim/DOWNSAMPL[1]);
    zdim=(newsize[2]=zdim/DOWNSAMPL[2]);
    newdata=downsample(data,size,newsize);  
    free(data);
    printf("%s: Doing isosurfacing\n",MY_NAME);
    if (iso_surface(newdata, xdim, ydim, zdim, threshold) == -1) {
      printf("%s: error from iso_surface\n", MY_NAME);
      exit(1);
    }
  }
  else {
    /* just plain isosurfacing! */
    printf("%s: Doing isosurfacing\n",MY_NAME);
      if (iso_surface(data, xdim, ydim, zdim, threshold) == -1) {
	printf("%s: error from iso_surface\n", MY_NAME);
      }
    free(data);
  }

  tid3=clock();
  /* polygon reduction */
  reduce();
  tid6=clock();

#ifdef TIME
  if (VERBOSE) {
    printf("%s: <Runtime (secs)> \n",MY_NAME);
    printf("%s:   filtering:       %7.2f\n",MY_NAME, (tid2-tid1)/(float)CLOCKS_PER_SEC);
    printf("%s:   iso-surfacing:   %7.2f\n",MY_NAME, (tid3-tid2)/(float)CLOCKS_PER_SEC);
    printf("%s:   poly. init:      %7.2f\n",MY_NAME, (tid4-tid3)/(float)CLOCKS_PER_SEC);
    printf("%s:   poly. reducing:  %7.2f\n",MY_NAME, (tid5-tid4)/(float)CLOCKS_PER_SEC);
    printf("%s:   total:           %7.2f\n",MY_NAME, (tid6-tid1)/(float)CLOCKS_PER_SEC);
  }
#endif

  printf("%s: done...\n\n",MY_NAME);
  exit(0);
}




















